﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Drawing;
using System.Runtime;
using System.Xml;
using ScriptPortal.Vegas;

namespace Test_Script
{
    public class Class1
    {
        public Vegas myVegas;

        public void Main(Vegas vegas)
        {
            myVegas = vegas;

            placeRegionsAroundAudio();
            checkOverlappedRegions();
            reverseRegions();
            closeGaps();
        }

        private void placeRegionsAroundAudio()
        {
            foreach (Track myTrack in myVegas.Project.Tracks)
            {
                foreach (TrackEvent evnt in myTrack.Events)
                {
                    AddRegion(evnt);
                }
            }
        }

        private void AddRegion(TrackEvent evnt)
        {
            bool eventUnderRegion = false;

            foreach (ScriptPortal.Vegas.Region myRegion in myVegas.Project.Regions)
            {
                Timecode regionEnd = myRegion.Position + myRegion.Length;

                if (evnt.Start >= myRegion.Position && evnt.Start <= regionEnd)
                {
                    eventUnderRegion = true;
                }
                if (evnt.End >= myRegion.Position && evnt.End <= regionEnd)
                {
                    eventUnderRegion = true;
                }

                if (eventUnderRegion)
                {
                    if (evnt.Start < myRegion.Position)
                    {
                        myRegion.Position = evnt.Start;
                        if (evnt.End > regionEnd)
                        {
                            myRegion.Length = evnt.End - myRegion.Position;
                        }
                        else
                        {
                            myRegion.Length = regionEnd - evnt.Start;
                        }
                        regionEnd = myRegion.Position + myRegion.Length;
                    }
                    if (evnt.End > regionEnd)
                    {
                        myRegion.Length = evnt.End - myRegion.Position;

                        if (myRegion.Index + 1 != myVegas.Project.Regions.Count)
                        {
                            if (myVegas.Project.Regions[myRegion.Index + 1].Position < evnt.End)
                            {
                                Timecode newEnd = myVegas.Project.Regions[myRegion.Index + 1].Position + myVegas.Project.Regions[myRegion.Index + 1].Length;
                                if (newEnd > myRegion.Position + myRegion.Length)
                                {
                                    myRegion.Length = newEnd - myRegion.Position;
                                }
                                myVegas.Project.Regions.Remove(myVegas.Project.Regions[myRegion.Index + 1]);
                            }
                        }
                    }
                    break;
                }
            }

            if (!eventUnderRegion)
            {
                addRegionToTimeline(evnt.Start, evnt.Length, "Audio Present");
            }
        }

        private void addRegionToTimeline(Timecode regStart, Timecode regLen, string title)
        {
            ScriptPortal.Vegas.Region newRegion = new ScriptPortal.Vegas.Region(regStart, regLen, title);
            myVegas.Project.Regions.Add(newRegion);
        }





        private void checkOverlappedRegions()
        {
            bool foundoverlap = true;

            while (foundoverlap)
            {
                for (int i = 0; i < myVegas.Project.Regions.Count; i++)
                {
                    foundoverlap = false;
                    ScriptPortal.Vegas.Region myRegion = myVegas.Project.Regions[i];

                    Timecode regStart = myRegion.Position;
                    Timecode regEnd = myRegion.Position + myRegion.Length;

                    foreach(ScriptPortal.Vegas.Region thisRegion in myVegas.Project.Regions)
                    {
                        if (myRegion.Index != thisRegion.Index)
                        {
                            Timecode thisStart = thisRegion.Position;
                            Timecode thisEnd = thisRegion.Position + thisRegion.Length;

                            if (regStart >= thisStart && regStart <= thisEnd)
                            {
                                foundoverlap = true;
                                if (thisEnd < regEnd)
                                {
                                    thisRegion.Length = regEnd - thisStart;
                                    myVegas.Project.Regions.Remove(myRegion);
                                    break;
                                }
                                else
                                {
                                    myVegas.Project.Regions.Remove(myRegion);
                                    break;
                                }
                            }
                        }
                    }

                    if (foundoverlap)
                    {
                        break;
                    }
                }
            }
        }

        private void reverseRegions()
        {
            ArrayList RegionStart = new ArrayList();
            ArrayList RegionLength = new ArrayList();

            foreach (ScriptPortal.Vegas.Region myRegion in myVegas.Project.Regions)
            {
                RegionStart.Add(myRegion.Position);
                RegionLength.Add(myRegion.Length);
            }

            for (int i = myVegas.Project.Regions.Count - 1; i >= 0; i--)
            {
                myVegas.Project.Regions.Remove(myVegas.Project.Regions[i]);
            }

            Timecode nextStart = new Timecode(0);
            for (int i = 0; i < RegionStart.Count; i++)
            {
                if (i == 0)
                {
                    if ((Timecode)RegionStart[i] > Timecode.FromFrames(0))
                    {
                        addRegionToTimeline(Timecode.FromFrames(0), (Timecode)RegionStart[i], "Audio Absent");
                    }
                }
                else
                {
                    addRegionToTimeline(nextStart, (Timecode)RegionStart[i] - nextStart, "Audio Absent");
                }
                nextStart = (Timecode)RegionStart[i] + (Timecode)RegionLength[i];
            }
        }

        private void closeGaps()
        {
            for (int i = myVegas.Project.Regions.Count - 1; i >= 0; i--)
            {
                Timecode regStart = myVegas.Project.Regions[i].Position;
                Timecode regLen = myVegas.Project.Regions[i].Length;

                foreach (Track myTrack in myVegas.Project.Tracks)
                {
                    foreach (TrackEvent evnt in myTrack.Events)
                    {
                        if (evnt.Start > regStart)
                        {
                            evnt.Start = evnt.Start - regLen;
                        }
                    }
                }
                myVegas.Project.Regions.Remove(myVegas.Project.Regions[i]);
            }
        }


    }
}

public class EntryPoint
{
    public void FromVegas(Vegas vegas)
    {
        Test_Script.Class1 test = new Test_Script.Class1();
        test.Main(vegas);
    }
}


